I suspect he used something slow to emulate sets in Perl 5. I get:
$ time python3 send.py
9567 + 1085 = 10652
real 0m16.388s
user 0m16.299s
sys 0m0.049s
$ time perl send.pl
9567 + 1085 = 10652
real 0m16.838s
user 0m16.732s
sys 0m0.063s
With: use strict;
use warnings;
my %digits = map { $_ => 1 } ( 0 .. 9 );
sub choose ($&) {
my ( $set, $sub ) = @_;
map { $sub->($_) } grep { $set->{$_} } keys %$set;
}
sub guard { $_[1]->() if $_[0] }
choose { %digits, map { $_ => 0 } ( 0 ) }, sub { my $s = $_[0];
choose { %digits, map { $_ => 0 } ( $s ) }, sub { my $e = $_[0];
choose { %digits, map { $_ => 0 } ( $s, $e ) }, sub { my $n = $_[0];
choose { %digits, map { $_ => 0 } ( $s, $e, $n ) }, sub { my $d = $_[0];
choose { %digits, map { $_ => 0 } ( 0, $s, $e, $n, $d ) }, sub { my $m = $_[0];
choose { %digits, map { $_ => 0 } ( $s, $e, $n, $d, $m ) }, sub { my $o = $_[0];
choose { %digits, map { $_ => 0 } ( $s, $e, $n, $d, $m, $o ) }, sub { my $r = $_[0];
choose { %digits, map { $_ => 0 } ( $s, $e, $n, $d, $m, $o, $r ) }, sub { my $y = $_[0];
my $send = $s . $e . $n . $d;
my $more = $m . $o . $r . $e;
my $money = $m . $o . $n . $e . $y;
guard( ($send + $more == $money), sub {
printf("%d + %d = %d\n", $send, $more, $money);
});
}}}}}}}};